package com.xy.service;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.xy.dto.DeviceErrorsRecordDto;
import com.xy.dto.DeviceInfoDto;
import com.xy.entity.DeviceErrorsRecord;
import com.xy.entity.DeviceInfo;
import com.xy.entity.SysCodeConfigureRedis;
import com.xy.mapper.DeviceErrorsRecordMapper;
import com.xy.utils.*;
import com.xy.utils.enums.SysCodeConfigureEnum;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.AllArgsConstructor;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;

import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import static com.xy.utils.Beans.copy;
import static com.xy.utils.PlusBeans.toIPage;
import static com.xy.utils.PlusBeans.toPageBean;

/**
 * <p>
 * 机器-故障表 服务实现类
 * </p>
 *
 * @author lijin
 * @since 2023-01-09
 */
@Service
@Api(tags = "机器-故障表")
@AllArgsConstructor(onConstructor_ = @Lazy)
public class DeviceErrorsRecordServiceImpl extends ServiceImpl<DeviceErrorsRecordMapper, DeviceErrorsRecord> implements DeviceErrorsRecordService {

    private DeviceInfoServiceImpl deviceInfoService;

    @Override
    @ApiOperation("添加")
    public R save(DeviceErrorsRecordDto.Save save) {
        DeviceInfoDto.Vo deviceInfo = deviceInfoService.obj(new DeviceInfoDto.Obj().setDeviceId(save.getDeviceId()).setIsSysinfo(true)).getData();
        if (deviceInfo == null) {
            return R.fail("设备不存在");
        }
        //添加故障记录
        save.setMercId(deviceInfo.getMercId());
        DeviceErrorsRecord deviceErrorsRecord = copy(DeviceErrorsRecord.class, save)
                .setCreateTime(LocalDateTime.now());
        save(deviceErrorsRecord);
        SysCodeConfigureRedis sysCodeConfigureRedis = SysCodeConfigureUtils.get(SysCodeConfigureEnum.D01.getCode(), save.getCode());
        //修改设备当前故障等级
        if (Emptys.check(sysCodeConfigureRedis.getExpand())) {
            DeviceInfo updateDeviceInfo = new DeviceInfo()
                    .setDeviceId(deviceInfo.getDeviceId())
                    .setFaultLevel(Integer.valueOf(sysCodeConfigureRedis.getExpand()));
            deviceInfoService.updateById(updateDeviceInfo);
        }
        return R.ok();
    }

    @Override
    @ApiOperation("分页查询")
    public R<PageBean<DeviceErrorsRecordDto.Vo>> page(@RequestBody DeviceErrorsRecordDto.Page page) {
        PageBean pageBean = page.getPage();
        LambdaQueryWrapper<DeviceErrorsRecord> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(page, DeviceErrorsRecord.class)
                .ge(DeviceErrorsRecord::getCreateTime, page.getBeginCreateTime())
                .le(DeviceErrorsRecord::getCreateTime, page.getEndCreateTime())
                .build()
                .orderByDesc(!Emptys.check(pageBean.getOrders()), DeviceErrorsRecord::getId);
        IPage<DeviceErrorsRecord> iPage = page(toIPage(pageBean), lambdaQueryWrapper);
        PageBean<DeviceErrorsRecordDto.Vo> voPageBean = toPageBean(DeviceErrorsRecordDto.Vo.class, iPage);
        //反显设备名称
        if (Emptys.check(voPageBean.getRecords())) {
            List<Long> deviceIdList = voPageBean.getRecords().stream().map(DeviceErrorsRecordDto.Vo::getDeviceId).collect(Collectors.toList());
            Map<Long, String> deviceMap = deviceInfoService.getDeviceNameList(new DeviceInfoDto.DeviceIdDto().setDeviceId(deviceIdList)).getData();
            Map<String, SysCodeConfigureRedis> stringSysCodeConfigureRedisMap = SysCodeConfigureUtils.get(SysCodeConfigureEnum.D01.getCode());
            voPageBean.getRecords().forEach(i -> {
                i.setDeviceName(deviceMap.get(i.getDeviceId()));
                SysCodeConfigureRedis sysCodeConfigureRedis = stringSysCodeConfigureRedisMap.get(i.getCode());
                if (sysCodeConfigureRedis != null) {
                    i.setSysCodeConfigureRedis(sysCodeConfigureRedis);
                }
            });
        }
        return R.ok(voPageBean);
    }

    @PostMapping("list")
    @ApiOperation("集合查询")
    public R<List<DeviceErrorsRecordDto.Vo>> list(@RequestBody DeviceErrorsRecordDto.SelectList selectList) {
        LambdaQueryWrapper<DeviceErrorsRecord> lambdaQueryWrapper = new MybatisPlusQuery().eqWrapper(selectList, DeviceErrorsRecord.class)
                .in(DeviceErrorsRecord::getId, selectList.getIds())
                .in(DeviceErrorsRecord::getDeviceId, selectList.getDeviceIds())
                .build();
        List<DeviceErrorsRecord> list = list(lambdaQueryWrapper);
        Map<String, SysCodeConfigureRedis> stringSysCodeConfigureRedisMap = SysCodeConfigureUtils.get(SysCodeConfigureEnum.D01.getCode());
        List<DeviceErrorsRecordDto.Vo> vos = copy(DeviceErrorsRecordDto.Vo.class, list);
        vos.forEach(vo -> {
            SysCodeConfigureRedis sysCodeConfigureRedis = stringSysCodeConfigureRedisMap.get(vo.getCode());
            if (sysCodeConfigureRedis != null) {
                vo.setSysCodeConfigureRedis(sysCodeConfigureRedis);
            }
        });
        return R.ok(vos);
    }

}